#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = __importDefault(require("process"));
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const execa_1 = __importDefault(require("execa"));
const archiver_1 = __importDefault(require("archiver"));
const crypto_random_string_1 = __importDefault(require("crypto-random-string"));
const commander_1 = __importDefault(require("commander"));
const globby_1 = __importDefault(require("globby"));
async function caxa({ input, output, command, force = true, exclude = [], filter = (() => {
    const pathsToExclude = globby_1.default.sync(exclude, {
        expandDirectories: false,
        onlyFiles: false,
    });
    return (path) => !pathsToExclude.includes(path);
})(), dedupe = true, prepareCommand, prepare = async (buildDirectory) => {
    if (prepareCommand === undefined)
        return;
    await execa_1.default.command(prepareCommand, { cwd: buildDirectory, shell: true });
}, includeNode = true, removeBuildDirectory = true, identifier = path_1.default.join(path_1.default.basename(path_1.default.basename(output, ".app"), ".exe"), crypto_random_string_1.default({ length: 10, type: "alphanumeric" }).toLowerCase()), }) {
    if (!(await fs_extra_1.default.pathExists(input)) || !(await fs_extra_1.default.lstat(input)).isDirectory())
        throw new Error(`The path to your application isn’t a directory: ‘${input}’.`);
    if ((await fs_extra_1.default.pathExists(output)) && !force)
        throw new Error(`Output already exists: ‘${output}’.`);
    if (process_1.default.platform === "win32" && !output.endsWith(".exe"))
        throw new Error("An Windows executable must end in ‘.exe’.");
    const buildDirectory = path_1.default.join(os_1.default.tmpdir(), "caxa/builds", crypto_random_string_1.default({ length: 10, type: "alphanumeric" }).toLowerCase());
    await fs_extra_1.default.copy(input, buildDirectory, { filter });
    if (dedupe)
        await execa_1.default("npm", ["dedupe", "--production"], { cwd: buildDirectory });
    await prepare(buildDirectory);
    if (includeNode) {
        const node = path_1.default.join(buildDirectory, "node_modules/.bin", path_1.default.basename(process_1.default.execPath));
        await fs_extra_1.default.ensureDir(path_1.default.dirname(node));
        await fs_extra_1.default.copyFile(process_1.default.execPath, node);
    }
    await fs_extra_1.default.ensureDir(path_1.default.dirname(output));
    await fs_extra_1.default.remove(output);
    if (output.endsWith(".app")) {
        if (process_1.default.platform !== "darwin")
            throw new Error("macOS Application Bundles (.app) are supported in macOS only.");
        await fs_extra_1.default.ensureDir(path_1.default.join(output, "Contents/Resources"));
        await fs_extra_1.default.move(buildDirectory, path_1.default.join(output, "Contents/Resources/application"));
        await fs_extra_1.default.ensureDir(path_1.default.join(output, "Contents/MacOS"));
        const name = path_1.default.basename(output, ".app");
        await fs_extra_1.default.writeFile(path_1.default.join(output, "Contents/MacOS", name), `#!/usr/bin/env sh\nopen "$(dirname "$0")/../Resources/${name}"`, { mode: 0o755 });
        await fs_extra_1.default.writeFile(path_1.default.join(output, "Contents/Resources", name), `#!/usr/bin/env sh\n${command
            .map((part) => `"${part.replace(/\{\{\s*caxa\s*\}\}/g, `$(dirname "$0")/application`)}"`)
            .join(" ")}`, { mode: 0o755 });
    }
    else {
        await fs_extra_1.default.copyFile(path_1.default.join(__dirname, "../stub"), output);
        await fs_extra_1.default.chmod(output, 0o755);
        const archive = archiver_1.default("tar", { gzip: true });
        const archiveStream = fs_extra_1.default.createWriteStream(output, { flags: "a" });
        archive.pipe(archiveStream);
        archive.directory(buildDirectory, false);
        await archive.finalize();
        // FIXME: Use ‘stream/promises’ when Node.js 16 lands, because then an LTS version will have the feature: await stream.finished(archiveStream);
        await new Promise((resolve, reject) => {
            archiveStream.on("finish", resolve);
            archiveStream.on("error", reject);
        });
        await fs_extra_1.default.appendFile(output, "\n" + JSON.stringify({ identifier, command }));
    }
    if (removeBuildDirectory)
        await fs_extra_1.default.remove(buildDirectory);
}
exports.default = caxa;
if (require.main === module)
    (async () => {
        await commander_1.default.program
            .version(require("../package.json").version)
            .requiredOption("-i, --input <input>", "The input directory to package.")
            .requiredOption("-o, --output <output>", "The path where the executable will be produced. On Windows must end in ‘.exe’. On macOS may end in ‘.app’ to generate a macOS Application Bundle.")
            .option("-f, --force", "[Advanced] Overwrite output if it exists.", true)
            .option("-F, --no-force")
            .option("-e, --exclude <path...>", "[Advanced] Paths to exclude from the build. The paths are passed to https://github.com/sindresorhus/globby and paths that match will be excluded.")
            .option("-d, --dedupe", "[Advanced] Run ‘npm dedupe --production’ on the build directory.", true)
            .option("-D, --no-dedupe")
            .option("-p, --prepare-command <command>", "[Advanced] Command to run on the build directory while packaging.")
            .option("-n, --include-node", "[Advanced] Copy the Node.js executable to ‘{{caxa}}/node_modules/.bin/node’.", true)
            .option("-N, --no-include-node")
            .option("-b, --remove-build-directory", "[Advanced] Remove the build directory after the build.", true)
            .option("-B, --no-remove-build-directory")
            .option("--identifier <identifier>", "[Advanced] Build identifier, which is the path in which the application will be unpacked.")
            .arguments("<command...>")
            .description("Package Node.js applications into executable binaries.", {
            command: "The command to run and optional arguments to pass to the command every time the executable is called. Paths must be absolute. The ‘{{caxa}}’ placeholder is substituted for the folder from which the package runs. The ‘node’ executable is available at ‘{{caxa}}/node_modules/.bin/node’. Use double quotes to delimit the command and each argument.",
        })
            .addHelpText("after", `
Examples:

  Windows:
  > caxa --input "examples/echo-command-line-parameters" --output "echo-command-line-parameters.exe" -- "{{caxa}}/node_modules/.bin/node" "{{caxa}}/index.js" "some" "embedded arguments" "--an-option-thats-part-of-the-command"

  macOS/Linux:
  $ caxa --input "examples/echo-command-line-parameters" --output "echo-command-line-parameters" -- "{{caxa}}/node_modules/.bin/node" "{{caxa}}/index.js" "some" "embedded arguments" "--an-option-thats-part-of-the-command"

  macOS (Application Bundle):
  $ caxa --input "examples/echo-command-line-parameters" --output "Echo Command Line Parameters.app" -- "{{caxa}}/node_modules/.bin/node" "{{caxa}}/index.js" "some" "embedded arguments" "--an-option-thats-part-of-the-command"
`)
            .action(async (command, { input, output, force, exclude = [], dedupe, prepareCommand, includeNode, removeBuildDirectory, identifier, }) => {
            try {
                await caxa({
                    input,
                    output,
                    command,
                    force,
                    exclude,
                    dedupe,
                    prepareCommand,
                    includeNode,
                    removeBuildDirectory,
                    identifier,
                });
            }
            catch (error) {
                console.error(error.message);
                process_1.default.exit(1);
            }
        })
            .parseAsync();
    })();
//# sourceMappingURL=index.js.map